/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    robdekon
 * @author     Christoph Pohl ( christoph dot pohl at kit dot edu )
 * @date       06.09.22
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include <RobotAPI/libraries/aron_component_config/PropertyDefinitionVisitors.h>
#include <RobotAPI/libraries/aron_component_config/RemoteGui.h>
#include <ArmarXCore/libraries/DecoupledSingleComponent/Decoupled.h>
#include "AronComponentConfigExample.h"

namespace armarx
{

    void
    AronComponentConfigExample::onInitComponent()
    {

    }

    void
    AronComponentConfigExample::onConnectComponent()
    {
        remote_gui_plugin_->createOrUpdateTab(aron_component_config_plugin_->buildRemoteGui("Config"),
                                              [this](armarx::RemoteGui::TabProxy& prx)
                                              {
                                                  prx.receiveUpdates();
                                                  std::lock_guard lock(write_mutex);
                                                  if (aron_component_config_plugin_->updateRemoteGui(prx))
                                                  {
                                                      remote_gui_plugin_->createOrUpdateTab("",
                                                                                            aron_component_config_plugin_
                                                                                                    ->buildRemoteGui(
                                                                                                            "Config"));
                                                  }
                                                  prx.sendUpdates();
                                              });
        periodicTask = new SimplePeriodicTask<>([&, this]
                                                {
                                                    std::lock_guard lock(write_mutex);
                                                    ARMARX_TRACE;
                                                    auto& config =
                                                            aron_component_config_plugin_->config_.getWriteBuffer();
//                                                    ARMARX_INFO << VAROUT(config.orientation) << VAROUT(config.pose) << VAROUT(config.position);
                                                    config.intMember--;
                                                    aron_component_config_plugin_->config_.commitWrite();
                                                }, 1000);
        periodicTask->start();
    }

    void
    AronComponentConfigExample::onDisconnectComponent()
    {
    }

    void
    AronComponentConfigExample::onExitComponent()
    {
    }

    armarx::PropertyDefinitionsPtr
    AronComponentConfigExample::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs = new AronComponentConfigExamplePropertyDefinitions(getConfigIdentifier());
        auto& config = aron_component_config_plugin_->config_.getWriteBuffer();
        config.boolMember = true;
        config.floatMember = 100.0f;
        config.intMember = 1000;
        config.stringMember = "initial";
        config.longMember = 0;
        config.subMember
              .subsubMember
              .intListMember
              .insert(config.subMember.subsubMember.intListMember.end(), {1, 2, 3, 4, 5, 6});
        config.subMember
              .subsubMember
              .stringListMember
              .insert(config.subMember.subsubMember.stringListMember.end(), {"a", "b", "c", "d", "e"});
        config.subMember.subsubMember.intDictMember.emplace("int1", 1);
        config.subMember.subsubMember.intDictMember.emplace("int2", 2);
        config.subMember.subsubMember.stringDictMember.emplace("string1", "blub");
        config.subMember.subsubMember.stringDictMember.emplace("string2", "duh");
        aron_component_config_plugin_->config_.commitWrite();
        return defs;
    }

    std::string
    AronComponentConfigExample::getDefaultName() const
    {
        return "ComponentConfigTest";
    }

    AronComponentConfigExample::AronComponentConfigExample()
    {
        addPlugin(remote_gui_plugin_, "");
        addPlugin(aron_component_config_plugin_, "");
    }

    AronComponentConfigExamplePropertyDefinitions::AronComponentConfigExamplePropertyDefinitions(std::string prefix) :
            armarx::ComponentPropertyDefinitions(std::move(prefix))
    {
    }
}