#include "SimpleReaderBase.h"

#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>
#include <RobotAPI/libraries/armem/core/error.h>

namespace armarx::armem::client::util
{

    SimpleReaderBase::SimpleReaderBase()
    {
    }

    void
    SimpleReaderBase::registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def)
    {
        ARMARX_DEBUG << "Writer: registerPropertyDefinitions";

        const std::string prefix = propertyPrefix();

        if (not props.has_value())
        {
            props = defaultProperties();
        }

        def->optional(props->memoryName, prefix + "Memory");
        def->optional(props->coreSegmentName, prefix + "CoreSegment");
    }

    void
    SimpleReaderBase::connect(armarx::armem::client::MemoryNameSystem& mns)
    {
        // Wait for the memory to become available and add it as dependency.
        ARMARX_IMPORTANT << "SimpleReaderBase: Waiting for memory '" << properties().memoryName
                         << "' ...";
        try
        {
            memoryReaderClient = mns.useReader(MemoryID().withMemoryName(properties().memoryName));
            ARMARX_IMPORTANT << "SimpleReaderBase: Connected to memory '" << properties().memoryName
                             << "'";
        }
        catch (const armem::error::CouldNotResolveMemoryServer& e)
        {
            ARMARX_ERROR << e.what();
            return;
        }
    }

    const armem::client::Reader&
    SimpleReaderBase::memoryReader() const
    {
        return memoryReaderClient;
    }

    const SimpleReaderBase::Properties&
    SimpleReaderBase::properties() const
    {
        if (not props.has_value())
        {
            const_cast<std::optional<Properties>&>(props) = defaultProperties();
        }
        return props.value();
    }

    void
    SimpleReaderBase::setProperties(const Properties& p)
    {
        props = p;
    }

} // namespace armarx::armem::client::util
