#pragma once

#include <string>
#include <vector>

#include <ArmarXCore/core/time/DateTime.h>
#include <ArmarXCore/core/time/ice_conversions.h>

#include <RobotAPI/interface/skills/SkillProviderInterface.h>
#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>

#include "SkillID.h"
#include "SkillParameterization.h"

namespace armarx
{
    namespace skills
    {
        struct SkillExecutionID
        {
            bool
            operator==(const SkillExecutionID& other) const
            {
                return this->toString() == other.toString();
            }

            bool
            operator<(const SkillExecutionID& other) const
            {
                return this->toString() < other.toString();
            }

            bool
            operator<=(const SkillExecutionID& other) const
            {
                return this->toString() <= other.toString();
            }

            std::string
            toString() const
            {
                return skillId.toString() + " requested by " + executorName + " at " +
                       executionStartedTime.toDateTimeString();
            }

            skills::manager::dto::SkillExecutionID toManagerIce() const;

            skills::provider::dto::SkillExecutionID toProviderIce() const;

            static SkillExecutionID FromIce(const skills::manager::dto::SkillExecutionID&);

            static SkillExecutionID FromIce(const skills::provider::dto::SkillExecutionID&,
                                            const std::optional<skills::ProviderID>& providerName);


            SkillID skillId;
            std::string executorName;
            armarx::core::time::DateTime executionStartedTime;
        };

    } // namespace skills
} // namespace armarx
