#pragma once

#include <RobotAPI/libraries/skills/provider/SpecializedSkill.h>

namespace armarx
{
    namespace skills
    {
        template <class AronT>
        class SimpleSpecializedSkill : public SpecializedSkill<AronT>
        {
        public:
            using Base = SpecializedSkill<AronT>;
            using ParamType = AronT;

            using Base::Base;

            struct SpecializedInitInput
            {
                std::string executorName;
                AronT parameters;
            };

            struct SpecializedMainInput
            {
                std::string executorName;
                AronT parameters;
                Skill::CallbackT callback;
            };

            struct SpecializedExitInput
            {
                std::string executorName;
                AronT parameters;
            };


        protected:
            // legacy methods
            virtual Skill::InitResult
            init(const SpecializedInitInput& in)
            {
                return Skill::InitResult{.status = TerminatedSkillStatus::Succeeded};
            }

            virtual Skill::MainResult
            main(const SpecializedMainInput& in)
            {
                ARMARX_IMPORTANT << "Dummy executing skill '" << this->description.skillId
                                 << "'. Please overwrite this method.";
                return Skill::MainResult{.status = TerminatedSkillStatus::Succeeded,
                                         .data = nullptr};
            }

            virtual Skill::ExitResult
            exit(const SpecializedExitInput& in)
            {
                return Skill::ExitResult{.status = TerminatedSkillStatus::Succeeded};
            }

            Skill::InitResult
            init() final
            {
                SpecializedInitInput i;
                i.executorName = this->executorName;
                i.parameters = this->getParameters();
                return this->init(i);
            }

            Skill::MainResult
            main() final
            {
                SpecializedMainInput i;
                i.executorName = this->executorName;
                i.callback = this->callback;
                i.parameters = this->getParameters();
                return this->main(i);
            }

            Skill::ExitResult
            exit() final
            {
                SpecializedExitInput i;
                i.executorName = this->executorName;
                i.parameters = this->getParameters();
                return this->exit(i);
            }
        };
    } // namespace skills
} // namespace armarx
